@@ -1,5 +1,5 @@
 package PAR;
-$PAR::VERSION = '0.991';
+$PAR::VERSION = '0.993';
 
 use 5.006;
 use strict;
@@ -33,10 +33,6 @@ use PAR::SetupTemp;
 
 PAR - Perl Archive Toolkit
 
-=head1 VERSION
-
-This document describes release 0.989_01 of PAR, released March 2, 2009.
-
 =head1 SYNOPSIS
 
 (If you want to make an executable that contains all module, scripts and
@@ -285,7 +281,15 @@ A par repository (exclusive of file)
 
 =item fallback
 
-Search the system @INC before the par.
+Search the system C<@INC> before the par.
+
+Off by default for loading F<.par> files via C<file => ...>.
+On by default for PAR repositories.
+
+To prefer loading modules from a repository over the locally
+installed modules, you can load the repository as follows:
+
+  use PAR { repository => 'http://foo/bar/', fallback => 0 };
 
 =item run
 
@@ -305,15 +309,16 @@ shared object file..."
 
 =cut
 
-use vars qw(@PAR_INC);              # explicitly stated PAR library files (prefered)
+use vars qw(@PAR_INC);              # explicitly stated PAR library files (preferred)
 use vars qw(@PAR_INC_LAST);         # explicitly stated PAR library files (fallback)
 use vars qw(%PAR_INC);              # sets {$par}{$file} for require'd modules
 use vars qw(@LibCache %LibCache);   # I really miss pseudohash.
 use vars qw($LastAccessedPAR $LastTempFile);
 use vars qw(@RepositoryObjects);    # If we have PAR::Repository::Client support, we
                                     # put the ::Client objects in here.
-use vars qw(@UpgradeRepositoryObjects); # If we have PAR::Repository::Client's in upgrade mode
-                                        # put the ::Client objects in here *as well*.
+use vars qw(@PriorityRepositoryObjects); # repositories which are preferred over local stuff
+use vars qw(@UpgradeRepositoryObjects);  # If we have PAR::Repository::Client's in upgrade mode
+                                         # put the ::Client objects in here *as well*.
 use vars qw(%FileCache);            # The Zip-file file-name-cache
                                     # Layout:
                                     # $FileCache{$ZipObj}{$FileName} = $Member
@@ -544,7 +549,11 @@ sub _import_repository {
         );
     }
 
-    push @RepositoryObjects, $obj;
+    if (exists($opt->{fallback}) and not $opt->{fallback}) {
+        push @PriorityRepositoryObjects, $obj; # repository beats local stuff
+    } else {
+        push @RepositoryObjects, $obj; # local stuff beats repository
+    }
     # these are tracked separately so we can check for upgrades early
     push @UpgradeRepositoryObjects, $obj if $opt->{upgrade};
 
@@ -631,6 +640,34 @@ sub _run_member {
     }
 }
 
+sub _run_external_file {
+    my $filename = shift;
+    my $clear_stack = shift;
+    require 5.008;
+    open my $ffh, '<', $filename
+      or die "Can't open perl script \"$filename\": $!";
+
+    my $clearstack = '';
+    if (defined &Internals::PAR::CLEARSTACK and $clear_stack) {
+        $clear_stack = "Internals::PAR::CLEARSTACK();\n";
+    }
+    my $string = "package main; shift \@INC;\n$clearstack#line 1 \"$filename\"\n"
+                 . do { local $/ = undef; <$ffh> };
+    close $ffh;
+
+    open my $fh, '<', \$string
+      or die "Can't open file handle to string: $!";
+
+    unshift @INC, sub { $fh };
+
+    $ENV{PAR_0} = $filename; # for Pod::Usage
+    { do 'main';
+      CORE::exit($1) if ($@ =~/^_TK_EXIT_\((\d+)\)/);
+      die $@ if $@;
+      exit;
+    }
+}
+
 # extract the contents of a .par (or .exe) or any
 # Archive::Zip handle to the PAR_TEMP/inc directory.
 # returns that directory.
@@ -715,7 +752,7 @@ sub _extract_inc {
 sub find_par {
     my @args = @_;
 
-    # if there are repositories win upgrade mode, check them
+    # if there are repositories in upgrade mode, check them
     # first. If so, this is expensive, of course!
     if (@UpgradeRepositoryObjects) {
         my $module = $args[1];
@@ -732,16 +769,39 @@ sub find_par {
                 # Then, realize that if you pass undef for the file handle, perl (5.8.9)
                 # does NOT use the subroutine. Thus the hacky GLOB ref.
                 my $line = 1;
+                no warnings;
                 return (\*I_AM_NOT_HERE, sub {$line ? ($_="1;",$line=0,return(1)) : ($_="",return(0))});
             }
 
-            if ($local_file) {
-                return _find_par_internals([$PAR_INC_LAST[-1]], @args);
-            }
+            # Note: This is likely not necessary as the module has been installed
+            # into the system by upgrade_module if it was available at all.
+            # If it was already loaded, this will not be reached (see return right above).
+            # If it could not be loaded from the system and neither found in the repository,
+            # we simply want to have the normal error message, too!
+            #
+            #if ($local_file) {
+            #    # XXX load with fallback - is that right?
+            #    return _find_par_internals([$PAR_INC_LAST[-1]], @args);
+            #}
         }
     }
+    my $rv = _find_par_internals(\@PAR_INC, @args);
 
-    return _find_par_internals(\@PAR_INC, @args);
+    return $rv if defined $rv or not @PriorityRepositoryObjects;
+
+    # the repositories that are prefered over locally installed modules
+    my $module = $args[1];
+    $module =~ s/\.pm$//;
+    $module =~ s/\//::/g;
+    foreach my $client (@PriorityRepositoryObjects) {
+        my $local_file = $client->get_module($module, 0); # 1 == fallback
+        if ($local_file) {
+            # Not loaded as fallback (cf. PRIORITY) thus look at PAR_INC
+            # instead of PAR_INC_LAST
+            return _find_par_internals([$PAR_INC[-1]], @args);
+        }
+    }
+    return();
 }
 
 # This is the hook placed in @INC for loading PAR's
@@ -763,6 +823,7 @@ sub find_par_last {
     foreach my $client (@RepositoryObjects) {
         my $local_file = $client->get_module($module, 1); # 1 == fallback
         if ($local_file) {
+            # Loaded as fallback thus look at PAR_INC_LAST
             return _find_par_internals([$PAR_INC_LAST[-1]], @args);
         }
     }
@@ -770,14 +831,6 @@ sub find_par_last {
 }
 
 
-# This is a conjunction of the early find_par and the late
-# find_par_last. It's called by PAR::Heavy for Dynaloader stuff.
-sub _find_par_any {
-    return _find_par_internals([@PAR_INC, @PAR_INC_LAST], @_);
-}
-
-
-
 # This routine implements loading modules from PARs
 # both for loading PARs preferably or as fallback.
 # To distinguish the cases, the first parameter should
@@ -788,7 +841,7 @@ sub _find_par_internals {
     my $scheme;
     foreach (@$INC_ARY ? @$INC_ARY : @INC) {
         my $path = $_;
-        if ($[ < 5.008001) {
+        if ($] < 5.008001) {
             # reassemble from "perl -Ischeme://path" autosplitting
             $path = "$scheme:$path" if !@$INC_ARY
                 and $path and $path =~ m!//!
